// life -- parallel, 2-dimensional, cellular automaton.
//   according to the rules of the game of life.
//   written using the Data-Parallel Programming Library for Education (DAPPLE)
//
//  Each cell in the array is either occupied or not.  At each iteration
// (generation), every cell counts its neighbors.  This is done for all 
// cells in parallel.  Then, simple rules determine which cells are occupied
// in the next generation.  After each generation we map the array to a 
// character array, and print it.  Boundaries are assumed to have unoccupied
// neighbors off the edge, rather than cyclic boundary conditions.
//
// David Kotz 1994-95

// $Id: life2.cc,v 1.3 95/12/17 00:08:16 dfk release Locker: dfk $

#include <stdlib.h>
#include <iostream.h>
#include "dapple.h"

int
main()
{
    int n, m;			// n rows, m columns
    int gen = 0;		// generation number
    
    cout << "Enter number of rows:" << endl;
    cin >> n;
    cout << "Enter number of columns:" << endl;
    cin >> m;
    
    charMatrix printable(n,m);
    
    cout << "Enter initial state (" << n << " rows by " << m << " columns, ";
    cout << "using 'O' for a live cell, '.' for an empty cell):" << endl;
    cin >> printable;
    
    cout << "Initial state:" << endl;
    cout << printable << endl;
    char newline;
    cin.get(newline);
    
    // note that because we use shift(), we assume boundaries of 0
    // if we used rotate, we would get periodic boundary conditions (torus)
    while (true) {
	// 1 means life there, 0 means nothing
	intMatrix cells(n,m), neighbors(n,m);

	ifp (printable == '.')
	  cells = 0;
	else
	  cells = 1;
	
	neighbors = (  shift(cells,  0, -1) 
		     + shift(cells,  0,  1)
		     + shift(cells, -1, -1)
		     + shift(cells, -1,  0)
		     + shift(cells, -1,  1)
		     + shift(cells,  1, -1)
		     + shift(cells,  1,  0)
		     + shift(cells,  1,  1));
	
	// The Rules:
	// If an organism has 0 or 1 neighbors, it dies of loneliness.
	// If an organism has 2 or 3 neighbors, it survives.
	// If an organism has 4 or more neighbors, it dies of overcrowding.
	// If an empty cell has exactly 3 neighbors, a new organism is born.
	
	ifp (neighbors != 2)
	  ifp (neighbors == 3)
	    cells = 1;		// either existing survives, or new is born
	  else
	    cells = 0;		// either lonely or overcrowded
	else
	  ; // no change with two neighbors
	
	cout << "Generation " << ++gen << ", " << sum(cells) << " alive" << endl;
	ifp (cells == 1)
	  printable = 'O';
	else
	  printable = '.';
	cout << printable << endl;
	
	cout << "press return:";
	char newline;
	cin.get(newline);
    }
    
    return(0);
}
