// strings -- search for a short string in a longer string
//   written using the Data-Parallel Programming Library for Education (DAPPLE)
//
// This program reads a long string into a charVector, and then a short string
// into a regular C string.  It prints out the position of all occurrences of
// the short string in the long string.
//
// David Kotz 1995

// $Id: strings.cc,v 1.3 95/12/17 00:08:28 dfk release Locker: dfk $

#include <stdlib.h>
#include <iostream.h>
#include <string.h>
#include "dapple.h"

const int TEXTLEN = 100;		// limit on length of long string, presumably large
const int PATLEN = 20;			// length of short pattern string

int
main()
{
	char textstring[TEXTLEN+1];		// the short string
	char newline;
	
	cout << "Please enter the long text, against which you will search" << endl;
	cout << "(limit " << TEXTLEN << " characters):" << endl;
	cin.get(textstring, TEXTLEN+1, '\n');
	cin.get(newline);
	int length = strlen(textstring);	// actual length of the text
	
	// ok, now declare some vectors to fit
	charVector text(length, textstring);	// the long string
	CONST intVector VP(length, Identity);	// a vector whose values are same as VP#
	booleanVector candidate(length);	// which positions might have the pattern?

	// lower-case the text
	ifp (text >= 'A' && text <= 'Z')
		text = text - 'A' + 'a';
	
	char pattern[PATLEN+1];			// the short string
	cout << "Please enter the pattern string, to search for (limit " 
		<< PATLEN << " characters):" << endl;
	cin.get(pattern, PATLEN+1, '\n');
	cin.get(newline);

	// lower-case the pattern
	for (char *p = pattern; *p != '\0'; p++)
		if (*p >= 'A' && *p <= 'Z')
			*p = *p - 'A' + 'a';
	
	// now conduct the search
	// initially all positions are candidates
	candidate = true;
	for (p = pattern; *p != '\0'; p++) {
		// at each candidate position, compare the pattern character against the text
		ifp (candidate && (text != *p))
			candidate = false;			// no match; no longer a candidate

		// the next character should be at the next VP to the right, 
		// so we make that VP a candidate
		candidate = shift(candidate, 1);
	}
	
	if (any(candidate)) {
		cout << "Pattern '" << pattern << "' found at positions ";
		// note that candidate is true at all VPs just *past* the pattern
		// so if we shift it back by the length of the pattern, 
		// it will be true at all VPs where the pattern begins.
		ifp (shift(candidate, -strlen(pattern)))
			cout << VP << endl;
	} else
		cout << "Pattern '" << pattern << "' not found." << endl;

    return(0);
}
