// @TITLE "tester.cc -- test program"
//
// tester -- test driver for DAPPLE
//
// David Kotz 1994

// $Id: tester.cc,v 2.16 95/12/17 00:08:14 dfk release Locker: dfk $

#include <stdlib.h>
#include <iostream.h>
#include "dapple.h"

const int N = 5;	// Vectors are size N
const int rows = N;	// Matrices are size N by M
const int cols = 4;	// Matrices are size N by M

// which VP am I? useful for VP-specific math
const intVector VP(N, Identity);
// which row or column am I? useful for VP-specific math
const intMatrix VProw(rows, cols, IdentityR);
const intMatrix VPcol(rows, cols, IdentityC);

static int square(const int x) { return (x*x); }

int array[N] = {3, 4, 0, -3, 2};
int P_array[N] = {4, 2, 0, 1, 3}; // a permutation

int
main()
{
    intVector A(N, array), B(N, 4), C(N);
    const intVector P(N, P_array);
    intMatrix M(rows, cols);
    int x;
    boolean b;

    cout << "Vector size is " << N << endl;

    cout << "A: " << A << endl;

    cout << "B: " << B << endl;

    B = 3;
    cout << "B = 3: " << B << endl;

    C = apply(square, A);
    cout << "C = apply(square, A): " << C  << endl;

    C = plus_scan(A);
    cout << "C = plus_scan(A): " << C  << endl;

    C = max_scan(A);
    cout << "C = max_scan(A): " << C  << endl;

    C = min_scan(A);
    cout << "C = min_scan(A): " << C  << endl;

    cout << "booleanVector(A): " << booleanVector(A)  << endl;

    C = or_scan(booleanVector(A));
    cout << "C = or_scan(booleanVector(A)): " << C  << endl;

    C = and_scan(booleanVector(A));
    cout << "C = and_scan(booleanVector(A)): " << C  << endl;

    x = sum(A);
    cout << "x = sum(A): ";
    cout << x;
    cout << endl;

    b = any(A);
    cout << "b = any(A): ";
    cout << b;
    cout << endl;

    b = all(A);
    cout << "b = all(A): ";
    cout << b;
    cout << endl;

    x = n_nonzeros(A);
    cout << "x = n_nonzeros(A): ";
    cout << x;
    cout << endl;

    x = max_value(A);
    cout << "x = max_value(A): ";
    cout << x;
    cout << endl;

    x = min_value(A);
    cout << "x = min_value(A): ";
    cout << x;
    cout << endl;

    x = max_index(A);
    cout << "x = max_index(A): ";
    cout << x;
    cout << endl;

    x = min_index(A);
    cout << "x = min_index(A): ";
    cout << x;
    cout << endl;

    cout << "A: " << A << endl;

    cout << "P: " << P << endl;

    C = permute(A, P);
    cout << "C = permute(A, P): " << C  << endl;

    C = shift(A, 1);
    cout << "C = shift(A, 1): " << C  << endl;

    C = shift(A, -3);
    cout << "C = shift(A, -3): " << C  << endl;

    C = shift(A, 10);
    cout << "C = shift(A, 10): " << C  << endl;

    C = rotate(A, 1);
    cout << "C = rotate(A, 1): " << C  << endl;

    C = rotate(A, -3);
    cout << "C = rotate(A, -3): " << C  << endl;

    C = rotate(A, 5);
    cout << "C = rotate(A, 5): " << C  << endl;

    C = rotate(A, -8);
    cout << "C = rotate(A, -8): " << C  << endl;

    cout << "VP: " << VP << endl;

    ifp (VP % 2 == 1)
      C = VP * 2;
    cout << "ifp(VP % 2) C = VP * 2: " << C  << endl;

    ifp (VP % 2 == 1) {
	C = VP * 2;
	ifp (C % 3 == 1)
	  C = 66;
	else
	  C = 77;
    }
    cout << "ifp(VP % 2) ...: " << C  << endl;

    ifp (VP % 2 == 1)
      C = rotate(VP, 2);
    cout << "ifp(VP % 2) C = rotate(VP, 2): " << C  << endl;

    ifp (VP % 2 == 0)
      C = rotate(VP, -1);
    cout << "ifp(VP % 2 == 0) C = rotate(VP, -1): " << C  << endl;

    ifp (VP % 2 == 0)
      C = shift(VP, -2);
    cout << "ifp(VP % 2 == 0) C = shift(VP, -2): " << C  << endl;

    ifp (VP % 2 == 1)
      C = shift(VP, 1);
    cout << "ifp(VP % 2) C = shift(VP, 1): " << C  << endl;

    C = pack(A);
    cout << "C = pack(A): " << C  << endl;

    C = A + B;
    cout << "C = A + B: " << C  << endl;

    C = A + 4;
    cout << "C = A + 4: " << C  << endl;

    C = 4 + A;
    cout << "C = 4 + A: " << C  << endl;

    C = A - B;
    cout << "C = A - B: " << C  << endl;

    C = A - 4;
    cout << "C = A - 4: " << C  << endl;

    C = 4 - A;
    cout << "C = 4 - A: " << C  << endl;

    C = A * B;
    cout << "C = A * B: " << C  << endl;

    C = A * 4;
    cout << "C = A * 4: " << C  << endl;

    C = 4 * A;
    cout << "C = 4 * A: " << C  << endl;

    C = -A / B;
    cout << "C = -A / B: " << C  << endl;

    C = +A / 4;
    cout << "C = +A / 4: " << C  << endl;

    C = A + B - 45 / B;
    cout << "C = A + B - 45 / B: " << C  << endl;

    C.store(array);
    cout << "C.store(array): ";
    cout << array[0];
    for (int i = 1; i < N; i++) 
      cout << "\t" << array[i];
    cout << endl;

    B.load(array);
    cout << "B.load(array): " << B << endl;

    cout << "before ifp " << endl;
    cout << "A: " << A << endl;

    ifp (A % 2 == 0) {
	A = B * VP / 3;
    } else
      A = 100;
    
    cout << "after ifp " << endl;
    cout << "A: " << A << endl;

    cout << "B: " << B << endl;

    cout << "C: " << C  << endl;

    C = B == C;
    cout << "B == C: " << C  << endl;

    C = A < -B;
    cout << "C = A < -B: " << C  << endl;

    B += 10;
    cout << "B += 10: " << B << endl;

    A /= 10;
    A *= -3;
    cout << "A /= 10; A *= -3: " << A << endl;

    C = A++;
    cout << "C = A++: " << C  << endl;

    cout << "A: " << A << endl;

    C = --A;
    cout << "C = --A: " << C  << endl;

    cout << "A: " << A << endl;

    A += B;
    cout << "A += B: " << A << endl;

    C = A >= B;
    cout << "C = A >= B: " << C  << endl;

    C = !(A >= B && -B != 9);
    cout << "C = !(A >= B && -B != 9): " << C  << endl;

    C = (A >= B || 14);
    cout << "C = (A >= B || 14): " << C  << endl;

    cout << "A: " << A << endl;
    cout << "B: " << B << endl;
    cout << "C: " << C << endl;
    
    C = A += B;
    cout << "C = A += B:" << endl;
    cout << " A: " << A << endl;
    cout << " B: " << B << endl;
    cout << " C: " << C << endl;
    
    cout << endl;

    cout << "VProw:" << endl;
    cout << VProw << endl;

    cout << "VPcol:" << endl;
    cout << VPcol << endl;

    M = VProw * VPcol;
    cout << "M = VProw * VPcol:" << endl;
    cout << M << endl;

    M = shift(VProw, -1, 0);
    cout << "M = shift(VProw, -1, 0):" << endl;
    cout << M << endl;

    M = rotate(VPcol, 0, 2);
    cout << "M = rotate(VPcol, 0, 2):" << endl;
    cout << M << endl;

    M = rotate(VProw, 2, 0);
    cout << "M = rotate(VProw, 2, 0):" << endl;
    cout << M << endl;

    M = rotate(VPcol * VProw, 3, 2);
    cout << "M = rotate(VPcol * VProw, 3, 2):" << endl;
    cout << M << endl;

    M = shift(VPcol * VProw, 3, -2);
    cout << "M = shift(VPcol * VProw, 3, -2):" << endl;
    cout << M << endl;

    M = VProw * VPcol;
    cout << "M = VProw * VPcol:" << endl;
    cout << M << endl;
    cout << "M[3]: " << M[3] << endl;
    cout << "M[3][_]: " << M[3][_] << endl;
    cout << "M[_][2]: " << M[_][2] << endl;
    M[_][1] = A;
    cout << "M[_][1] = A;  M:" << endl;
    cout << M << endl;

    cout << "min_scan_rows(M):" << endl;
    cout << min_scan_rows(M) << endl;

    cout << "max_scan_cols(M):" << endl;
    cout << max_scan_cols(M) << endl;

    cout << "plus_scan_cols(M):" << endl;
    cout << plus_scan_cols(M) << endl;

    cout << "or_scan_cols(VProw % 2 == 1):" << endl;
    cout << or_scan_cols(VProw % 2 == 1) << endl;

    cout << "M[_][1] / 4: " << M[_][1] / 4 << endl;
    cout << "4 / M[_][1]: " << 4 / M[_][1] << endl;
    cout << "A / M[_][1]: " << A / M[_][1] << endl;
    cout << "M[_][1] / A: " << M[_][1] / A << endl;
    cout << "M[_][2] / M[_][1]: " << M[_][2] / M[_][1] << endl;

    return(0);
}
