// @TITLE "dapple.h: Interface to DAPPLE -- Data-Parallel Programming Library for Education"
//
//  DAPPLE -- Data-Parallel Programming Library for Education
//
//     This class library is intended to support a Vector type,
// and simple operations on vectors.  The model is designed to simulate
// a SIMD, data-parallel, programming environment.
//
// David Kotz 1994
//

// $Id: dapple.h,v 2.16 95/12/17 00:07:45 dfk release Locker: dfk $

#ifndef DAPPLE_H
#define DAPPLE_H

#include <stdlib.h>
#include <iostream.h>
#include "macros.h"
#include "errors.h"
#include "Shape.h"
#include "BigSmall.h"
// more at bottom

// These are handy definitions you may want in your program:
// which VP am I? useful for VP-specific math
//    CONST intVector VP(size, Identity);
// which row or column am I? useful for VP-specific math
//    CONST intMatrix VProw(nrows, ncols, IdentityR);
//    CONST intMatrix VPcol(nrows, ncols, IdentityC);

static inline int Identity(CONST int x) { return(x); }
static inline int IdentityR(CONST int r, CONST int c) { return(r); }
static inline int IdentityC(CONST int r, CONST int c) { return(c); }

////////////////////////////////////////
// @SUBTITLE "Collections of various types"
// Now we need to declare collections
// for each interesting element type.  Eventually
// this should be done with templates.  Unfortunately,
// all collections must be defined before any vectors
// may be defined, so we do this in two stages...

// boolean
#define Element boolean
#define Collection booleanCollection
#define Vector booleanVector
#define Matrix booleanMatrix
#define Slice booleanSlice
#define HalfSlice booleanHalfSlice
#define BOOLEAN_ELEMENTS
#  include "Collection.h"
#undef Element
#undef Collection
#undef Vector
#undef Matrix
#undef Slice
#undef HalfSlice
#undef BOOLEAN_ELEMENTS

// int
#define Element int
#define Collection intCollection
#define Vector intVector
#define Matrix intMatrix
#define Slice intSlice
#define HalfSlice intHalfSlice
#define INT_ELEMENTS
#  include "Collection.h"
#undef Element
#undef Collection
#undef Vector
#undef Matrix
#undef Slice
#undef HalfSlice
#undef INT_ELEMENTS

// char
#define Element char
#define Collection charCollection
#define Vector charVector
#define Matrix charMatrix
#define Slice charSlice
#define HalfSlice charHalfSlice
#define CHAR_ELEMENTS
#  include "Collection.h"
#undef Element
#undef Collection
#undef Vector
#undef Matrix
#undef Slice
#undef HalfSlice
#undef CHAR_ELEMENTS

// float
#define Element float
#define Collection floatCollection
#define Vector floatVector
#define Matrix floatMatrix
#define Slice floatSlice
#define HalfSlice floatHalfSlice
#define FLOAT_ELEMENTS
#  include "Collection.h"
#undef Element
#undef Collection
#undef Vector
#undef Matrix
#undef Slice
#undef HalfSlice
#undef FLOAT_ELEMENTS

// double
#define Element double
#define Collection doubleCollection
#define Vector doubleVector
#define Matrix doubleMatrix
#define Slice doubleSlice
#define HalfSlice doubleHalfSlice
#define DOUBLE_ELEMENTS
#  include "Collection.h"
#undef Element
#undef Collection
#undef Vector
#undef Matrix
#undef Slice
#undef HalfSlice
#undef DOUBLE_ELEMENTS

////////////////////////////////////////
// @SUBTITLE "Vector classes"

// boolean
#define Element boolean
#define Collection booleanCollection
#define Vector booleanVector
#define Matrix booleanMatrix
#define Slice booleanSlice
#define HalfSlice booleanHalfSlice
#define BOOLEAN_ELEMENTS
#  include "Vector.h"
#undef Element
#undef Collection
#undef Vector
#undef Matrix
#undef Slice
#undef HalfSlice
#undef BOOLEAN_ELEMENTS

// int
#define Element int
#define Collection intCollection
#define Vector intVector
#define Matrix intMatrix
#define Slice intSlice
#define HalfSlice intHalfSlice
#define INT_ELEMENTS
#  include "Vector.h"
#undef Element
#undef Collection
#undef Vector
#undef Matrix
#undef Slice
#undef HalfSlice
#undef INT_ELEMENTS

// char
#define Element char
#define Collection charCollection
#define Vector charVector
#define Matrix charMatrix
#define Slice charSlice
#define HalfSlice charHalfSlice
#define CHAR_ELEMENTS
#  include "Vector.h"
#undef Element
#undef Collection
#undef Vector
#undef Matrix
#undef Slice
#undef HalfSlice
#undef CHAR_ELEMENTS

// float
#define Element float
#define Collection floatCollection
#define Vector floatVector
#define Matrix floatMatrix
#define Slice floatSlice
#define HalfSlice floatHalfSlice
#define FLOAT_ELEMENTS
#  include "Vector.h"
#undef Element
#undef Collection
#undef Vector
#undef Matrix
#undef Slice
#undef HalfSlice
#undef FLOAT_ELEMENTS

// double
#define Element double
#define Collection doubleCollection
#define Vector doubleVector
#define Matrix doubleMatrix
#define Slice doubleSlice
#define HalfSlice doubleHalfSlice
#define DOUBLE_ELEMENTS
#  include "Vector.h"
#undef Element
#undef Collection
#undef Vector
#undef Matrix
#undef Slice
#undef HalfSlice
#undef DOUBLE_ELEMENTS

////////////////////////////////////////
// @SUBTITLE "Matrix and Slice classes"

// boolean
#define Element boolean
#define Collection booleanCollection
#define Vector booleanVector
#define Matrix booleanMatrix
#define Slice booleanSlice
#define HalfSlice booleanHalfSlice
#define BOOLEAN_ELEMENTS
#  include "Slice.h"
#  include "Matrix.h"
#undef Element
#undef Collection
#undef Vector
#undef Matrix
#undef Slice
#undef HalfSlice
#undef BOOLEAN_ELEMENTS

// int
#define Element int
#define Collection intCollection
#define Vector intVector
#define Matrix intMatrix
#define Slice intSlice
#define HalfSlice intHalfSlice
#define INT_ELEMENTS
#  include "Slice.h"
#  include "Matrix.h"
#undef Element
#undef Collection
#undef Vector
#undef Matrix
#undef Slice
#undef HalfSlice
#undef INT_ELEMENTS

// char
#define Element char
#define Collection charCollection
#define Vector charVector
#define Matrix charMatrix
#define Slice charSlice
#define HalfSlice charHalfSlice
#define CHAR_ELEMENTS
#  include "Slice.h"
#  include "Matrix.h"
#undef Element
#undef Collection
#undef Vector
#undef Matrix
#undef Slice
#undef HalfSlice
#undef CHAR_ELEMENTS

// float
#define Element float
#define Collection floatCollection
#define Vector floatVector
#define Matrix floatMatrix
#define Slice floatSlice
#define HalfSlice floatHalfSlice
#define FLOAT_ELEMENTS
#  include "Slice.h"
#  include "Matrix.h"
#undef Element
#undef Collection
#undef Vector
#undef Matrix
#undef Slice
#undef HalfSlice
#undef FLOAT_ELEMENTS

// double
#define Element double
#define Collection doubleCollection
#define Vector doubleVector
#define Matrix doubleMatrix
#define Slice doubleSlice
#define HalfSlice doubleHalfSlice
#define DOUBLE_ELEMENTS
#  include "Slice.h"
#  include "Matrix.h"
#undef Element
#undef Collection
#undef Vector
#undef Matrix
#undef Slice
#undef HalfSlice
#undef DOUBLE_ELEMENTS

//////////////////////////////

#include "ActiveSet.h"

#endif // DAPPLE_H
